@@ -34,7 +34,7 @@
 #include <wocky/wocky-pep-service.h>
 
 #include "gabble/connection.h"
-#include "capabilities.h"
+#include "gabble/capabilities.h"
 #include "error.h"
 #include "ft-manager.h"
 #include "jingle-factory.h"
@@ -81,6 +81,7 @@ typedef enum
   GABBLE_CONNECTION_FEATURES_INVISIBLE = 1 << 6,
   GABBLE_CONNECTION_FEATURES_GOOGLE_SHARED_STATUS = 1 << 7,
   GABBLE_CONNECTION_FEATURES_GOOGLE_QUEUE = 1 << 8,
+  GABBLE_CONNECTION_FEATURES_GOOGLE_SETTING = 1 << 9,
 } GabbleConnectionFeatures;
 
 typedef struct _GabbleConnectionPrivate GabbleConnectionPrivate;
@@ -203,8 +204,6 @@ typedef enum {
     GABBLE_CONNECTION_ALIAS_FROM_ROSTER
 } GabbleConnectionAliasSource;
 
-gchar *gabble_connection_get_full_jid (GabbleConnection *conn);
-
 WockyPorter *gabble_connection_dup_porter (GabbleConnection *conn);
 
 gboolean _gabble_connection_set_properties_from_account (
