@@ -176,6 +176,13 @@ gabble_im_channel_fill_immutable_properties (TpBaseChannel *chan,
       NULL);
 }
 
+static gchar *
+gabble_im_channel_get_object_path_suffix (TpBaseChannel *chan)
+{
+  return g_strdup_printf ("ImChannel%u",
+      tp_base_channel_get_target_handle (chan));
+}
+
 static void
 gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
 {
@@ -196,6 +203,7 @@ gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
   base_class->close = gabble_im_channel_close;
   base_class->fill_immutable_properties =
     gabble_im_channel_fill_immutable_properties;
+  base_class->get_object_path_suffix = gabble_im_channel_get_object_path_suffix;
 
   tp_message_mixin_init_dbus_properties (object_class);
 }
@@ -384,65 +392,17 @@ _gabble_im_channel_send_message (GObject *object,
     priv->send_nick = FALSE;
 }
 
-/**
- * _gabble_im_channel_receive
- *
- */
-void
-_gabble_im_channel_receive (GabbleIMChannel *chan,
-                            TpChannelTextMessageType type,
-                            TpHandle sender,
-                            const char *from,
-                            time_t timestamp,
-                            const gchar *id,
-                            const char *text,
-                            TpChannelTextSendError send_error,
-                            TpDeliveryStatus delivery_status,
-                            gint state)
+static TpMessage *
+build_message (
+    GabbleIMChannel *self,
+    TpChannelTextMessageType type,
+    time_t timestamp,
+    const char *text)
 {
-  GabbleIMChannelPrivate *priv;
-  TpBaseChannel *base_chan;
-  TpBaseConnection *base_conn;
-  TpMessage *msg;
-  gchar *tmp;
+  TpBaseChannel *base_chan = (TpBaseChannel *) self;
+  TpBaseConnection *base_conn = tp_base_channel_get_connection (base_chan);
+  TpMessage *msg = tp_cm_message_new (base_conn, 2);
 
-  g_assert (GABBLE_IS_IM_CHANNEL (chan));
-  priv = chan->priv;
-  base_chan = (TpBaseChannel *) chan;
-  base_conn = tp_base_channel_get_connection (base_chan);
-
-  if (send_error == GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
-    {
-      /* update peer's full JID if it's changed */
-      if (tp_strdiff (from, priv->peer_jid))
-        {
-          g_free (priv->peer_jid);
-          priv->peer_jid = g_strdup (from);
-        }
-
-      if (state == -1)
-        {
-          priv->chat_states_supported = CHAT_STATES_NOT_SUPPORTED;
-        }
-      else
-        {
-          _gabble_im_channel_state_receive (chan, state);
-        }
-    }
-  else
-    {
-      /* strip off the resource (if any), since we just failed to send to it */
-      char *slash = strchr (priv->peer_jid, '/');
-
-      if (slash != NULL)
-        *slash = '\0';
-
-      priv->chat_states_supported = CHAT_STATES_UNKNOWN;
-    }
-
-  msg = tp_cm_message_new (base_conn, 2);
-
-  /* Header */
   if (type != TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL)
     tp_message_set_uint32 (msg, 0, "message-type", type);
 
@@ -453,49 +413,133 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
   tp_message_set_string (msg, 1, "content-type", "text/plain");
   tp_message_set_string (msg, 1, "content", text);
 
-  if (send_error == GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
+  return msg;
+}
+
+/*
+ * _gabble_im_channel_receive:
+ * @chan: a channel
+ * @type: the message type
+ * @from: the full JID we received the message from
+ * @timestamp: the time at which the message was sent (not the time it was
+ *             received)
+ * @id: the id='' attribute from the <message/> stanza, if any
+ * @text: the plaintext body of the message
+ * @send_error: the reason why sending @text to @sender failed, or
+ *              GABBLE_TEXT_CHANNEL_SEND_NO_ERROR if this call is not to report
+ *              a failure to send.
+ * @delivery_status: if @send_error is GABBLE_TEXT_CHANNEL_SEND_NO_ERROR,
+ *                   ignored; else the delivery status to attach to the report.
+ * @state: a #TpChannelChatState, or -1 if there was no chat state in the
+ *         message.
+ *
+ * Shoves an incoming message into @chan, possibly updating the chat state at
+ * the same time; or maybe this is a delivery report? Who knows! It's a magical
+ * adventure.
+ */
+void
+_gabble_im_channel_receive (GabbleIMChannel *chan,
+                            TpChannelTextMessageType type,
+                            const char *from,
+                            time_t timestamp,
+                            const gchar *id,
+                            const char *text,
+                            gint state)
+{
+  GabbleIMChannelPrivate *priv;
+  TpBaseChannel *base_chan;
+  TpHandle peer;
+  TpMessage *msg;
+
+  g_assert (GABBLE_IS_IM_CHANNEL (chan));
+  priv = chan->priv;
+  base_chan = (TpBaseChannel *) chan;
+  peer = tp_base_channel_get_target_handle (base_chan);
+
+  /* update peer's full JID if it's changed */
+  if (tp_strdiff (from, priv->peer_jid))
     {
-      tp_cm_message_set_sender (msg, sender);
-      tp_message_set_int64 (msg, 0, "message-received", time (NULL));
-
-      if (id != NULL)
-        tp_message_set_string (msg, 0, "message-token", id);
-
-      tp_message_mixin_take_received (G_OBJECT (chan), msg);
+      g_free (priv->peer_jid);
+      priv->peer_jid = g_strdup (from);
     }
+
+  if (state == -1)
+    priv->chat_states_supported = CHAT_STATES_NOT_SUPPORTED;
   else
+    _gabble_im_channel_state_receive (chan, state);
+
+  msg = build_message (chan, type, timestamp, text);
+  tp_cm_message_set_sender (msg, peer);
+  tp_message_set_int64 (msg, 0, "message-received", time (NULL));
+
+  if (id != NULL)
+    tp_message_set_string (msg, 0, "message-token", id);
+
+  tp_message_mixin_take_received (G_OBJECT (chan), msg);
+}
+
+void
+_gabble_im_channel_report_delivery (
+    GabbleIMChannel *self,
+    TpChannelTextMessageType type,
+    time_t timestamp,
+    const gchar *id,
+    const char *text,
+    TpChannelTextSendError send_error,
+    TpDeliveryStatus delivery_status)
+{
+  GabbleIMChannelPrivate *priv;
+  TpBaseChannel *base_chan = (TpBaseChannel *) self;
+  TpBaseConnection *base_conn;
+  TpHandle peer;
+  TpMessage *msg, *delivery_report;
+  gchar *tmp;
+
+  g_return_if_fail (GABBLE_IS_IM_CHANNEL (self));
+  priv = self->priv;
+  peer = tp_base_channel_get_target_handle (base_chan);
+  base_conn = tp_base_channel_get_connection (base_chan);
+
+  if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
     {
-      TpMessage *delivery_report = tp_cm_message_new (base_conn, 1);
+      /* strip off the resource (if any), since we just failed to send to it */
+      char *slash = strchr (priv->peer_jid, '/');
 
-      tp_message_set_uint32 (delivery_report, 0, "message-type",
-          TP_CHANNEL_TEXT_MESSAGE_TYPE_DELIVERY_REPORT);
-      tp_cm_message_set_sender (delivery_report, sender);
-      tp_message_set_int64 (delivery_report, 0, "message-received",
-          time (NULL));
+      if (slash != NULL)
+        *slash = '\0';
 
-      tmp = gabble_generate_id ();
-      tp_message_set_string (delivery_report, 0, "message-token", tmp);
-      g_free (tmp);
-
-      tp_message_set_uint32 (delivery_report, 0, "delivery-status",
-          delivery_status);
-      tp_message_set_uint32 (delivery_report, 0, "delivery-error", send_error);
-
-      if (id != NULL)
-        tp_message_set_string (delivery_report, 0, "delivery-token", id);
-
-      /* We're getting a send error, so the original sender of the echoed
-       * message must be us! */
-      tp_cm_message_set_sender (msg, base_conn->self_handle);
-
-      /* Since this is a send error, we can trust the id on the message. */
-      if (id != NULL)
-        tp_message_set_string (msg, 0, "message-token", id);
-
-      tp_cm_message_take_message (delivery_report, 0, "delivery-echo", msg);
-
-      tp_message_mixin_take_received (G_OBJECT (chan), delivery_report);
+      priv->chat_states_supported = CHAT_STATES_UNKNOWN;
     }
+
+  msg = build_message (self, type, timestamp, text);
+  delivery_report = tp_cm_message_new (base_conn, 1);
+  tp_message_set_uint32 (delivery_report, 0, "message-type",
+      TP_CHANNEL_TEXT_MESSAGE_TYPE_DELIVERY_REPORT);
+  tp_cm_message_set_sender (delivery_report, peer);
+  tp_message_set_int64 (delivery_report, 0, "message-received",
+      time (NULL));
+
+  tmp = gabble_generate_id ();
+  tp_message_set_string (delivery_report, 0, "message-token", tmp);
+  g_free (tmp);
+
+  tp_message_set_uint32 (delivery_report, 0, "delivery-status",
+      delivery_status);
+  tp_message_set_uint32 (delivery_report, 0, "delivery-error", send_error);
+
+  if (id != NULL)
+    tp_message_set_string (delivery_report, 0, "delivery-token", id);
+
+  /* This is a delivery report, so the original sender of the echoed message
+   * must be us! */
+  tp_cm_message_set_sender (msg, base_conn->self_handle);
+
+  /* Since this is a delivery report, we can trust the id on the message. */
+  if (id != NULL)
+    tp_message_set_string (msg, 0, "message-token", id);
+
+  tp_cm_message_take_message (delivery_report, 0, "delivery-echo", msg);
+  tp_message_mixin_take_received (G_OBJECT (self), delivery_report);
 }
 
 /**
