@@ -2092,7 +2092,7 @@ gabble_jingle_session_terminate (GabbleJingleSession *sess,
 
           lm_message_node_add_child (r, reason_elt, NULL);
 
-          if (!CHECK_STR_EMPTY(text))
+          if (!tp_str_empty (text))
             lm_message_node_add_child (r, "text", text);
         }
 
