@@ -137,6 +137,7 @@ CONN_IFACE_CONTACT_LIST = CONN + '.Interface.ContactList'
 CONN_IFACE_CONTACT_GROUPS = CONN + '.Interface.ContactGroups'
 CONN_IFACE_CLIENT_TYPES = CONN + '.Interface.ClientTypes'
 CONN_IFACE_POWER_SAVING = CONN + '.Interface.PowerSaving'
+CONN_IFACE_CONTACT_BLOCKING = CONN + '.Interface.ContactBlocking'
 
 ATTR_CONTACT_CAPABILITIES = CONN_IFACE_CONTACT_CAPS + '/capabilities'
 
@@ -443,6 +444,14 @@ DELIVERY_REPORTING_SUPPORT_FLAGS_RECEIVE_SUCCESSES = 2
 DELIVERY_REPORTING_SUPPORT_FLAGS_RECEIVE_READ = 4
 DELIVERY_REPORTING_SUPPORT_FLAGS_RECEIVE_DELETED = 8
 
+DELIVERY_STATUS_UNKNOWN = 0
+DELIVERY_STATUS_DELIVERED = 1
+DELIVERY_STATUS_TEMPORARILY_FAILED = 2
+DELIVERY_STATUS_PERMANENTLY_FAILED = 3
+DELIVERY_STATUS_ACCEPTED = 4
+DELIVERY_STATUS_READ = 5
+DELIVERY_STATUS_DELETED = 6
+
 MEDIA_STREAM_ERROR_UNKNOWN = 0
 MEDIA_STREAM_ERROR_EOS = 1
 MEDIA_STREAM_ERROR_CODEC_NEGOTIATION_FAILED = 2
